#include "project.h"
#include "stdio.h"

#define CR '\r'
#define NL '\n'
#define BS '\b'


#define RX_BUF_LEN 64
volatile char rx_buf[RX_BUF_LEN] ;

#define STR_LEN 32
char str[STR_LEN+1] ; /* 1 byte for NULL */
void print(char *str)
{
    UART_UartPutString(str) ;
    while(UART_SpiUartGetTxBufferSize() > 0) ;
}

void printc(char c)
{
    UART_UartPutChar(c) ;
}

volatile int line_received = 0 ;
volatile int rx_write_index = 0 ;
int rx_read_index = 0 ;
int str_index = 0 ;

CY_ISR(uart_isr)
{
    uint32_t status ;
    uint8_t c ;
    
    status = UART_GetRxInterruptSource() ;
    if (status & UART_INTR_RX_NOT_EMPTY){
        c = UART_UartGetByte() ;
        if ((c != 0x00) && (c != 0xFF)) {
            rx_buf[rx_write_index] = c ;
            rx_write_index = (rx_write_index + 1) % RX_BUF_LEN ;
        }
    }
    UART_ClearRxInterruptSource(status) ;
}

int get_line(void)
{
    uint8_t c ;
    while((line_received == 0)&&(rx_read_index != rx_write_index)) {
        c = rx_buf[rx_read_index] ;
        switch(c) {
        case 0x00:
        case 0xFF:
            break ;
        case BS: /* backspace */
            if (str_index > 0) {
                str_index-- ;
                str[str_index] = 0 ;
                print("\b \b") ;
            }
            break ;
        case NL:
        case CR:
            printc(c) ;
            str[str_index] = 0 ;
            str_index = 0 ;
            line_received = 1 ;
            break ;
        default:
            printc(c) ;
            str[str_index] = c ;
            str_index++ ;
            if (str_index >= STR_LEN) { /* buffer overflow */
                str[STR_LEN] = 0 ; /* terminate the string */
                line_received = -1 ; /* not 0 but not 1 as this is error */
                str_index = 0 ;
            }  
        }
        rx_read_index = (rx_read_index + 1) % RX_BUF_LEN ;
    }
    return(line_received) ;
}

void prompt(void)
{
    print("> ") ;
}

int main(void)
{
    CyGlobalIntEnable; /* Enable global interrupts. */
    
    tty_rx_int_ClearPending() ;
    tty_rx_int_StartEx(uart_isr) ;

    UART_Start() ;
    
    print("\x1b[2J\x1b[;H") ; /* clear screen */
    print("UART interrupt test ") ;
    snprintf(str, STR_LEN, "(%s %s)\n", __DATE__, __TIME__) ;
    print(str) ;
    
    prompt() ;
    for(;;)
    {
        if (get_line()) { 
            print("Line: ") ;
            print(str) ;
            print("\n") ;
            line_received = 0 ;
            prompt() ;
        }
    }
}
